/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_appendix.h"
#include "ebi_error.h"

__EBI_BEGIN_NAMESPACE

Appendix::Appendix ()
{
  eb_initialize_appendix (&m_appendix);
}

Appendix::~Appendix ()
{
  close ();
}

EB_Appendix * Appendix::getAppendix ()
{
  return &m_appendix;
}

bool Appendix::findAppendix (std::string * pathName) const
{
  const char * c_names[] =
    {
      "_APPENDIX",
      "_appendix",
      "_Appendix",
      NULL
    };

  for (int i = 0; c_names[i] != NULL; ++i)
    {
      struct stat buf;
      if (stat (c_names[i], &buf) == 0)
	{
	  *pathName = c_names[i];
	  return true;
	}
    }

  pathName->erase ();
  return false;
}

bool Appendix::open ()
{
  std::string pathName;
  if (!findAppendix (&pathName))
    {
      return true;
    }

  eb_initialize_appendix (&m_appendix);

  Error::s_error = eb_bind_appendix (&m_appendix, pathName.c_str ());
  if (Error::s_error != EB_SUCCESS)
    {
      Error::outputMessageEB ("ϿǡǤޤ");
      return false;
    }

  eb_load_all_appendix_subbooks (&m_appendix);

  return true;
}

void Appendix::close ()
{
  if (eb_is_appendix_bound (&m_appendix))
    {
      eb_finalize_appendix (&m_appendix);
    }
}

bool Appendix::setSubbook (EB_Subbook_Code subbook)
{
  if (eb_is_appendix_bound (&m_appendix))
    {
      Error::s_error = eb_set_appendix_subbook (&m_appendix, subbook);
      if (Error::s_error != EB_SUCCESS)
	{
	  Error::outputMessageEB ("ϿǡǤޤ");
	  return false;
	}
    }

  return true;
}

__EBI_END_NAMESPACE

