/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_bitmapfont.h"
#include "ebi_error.h"

__EBI_BEGIN_NAMESPACE

BitmapFont::BitmapFont (const SIZE & size, const char * binary)
{
  m_bmp.planes = 1;
  m_bmp.pixbits = 0x0101;
  m_bmp.rowbytes = (size.h + 15) / 16 * 2;
  m_bmp.bounds = toRECT (0, 0, size.h, size.v);

  W bytes = m_bmp.rowbytes * size.v;
  m_bmp.baseaddr[0] = new UB[bytes];

  int srcRowbytes = (size.h + 7) / 8;
  if (m_bmp.rowbytes == srcRowbytes)
    {
      memcpy (m_bmp.baseaddr[0], binary, bytes);
    }
  else
    {
      const char *src = binary;
      UB *dest = m_bmp.baseaddr[0];
      for (int i = 0; i < size.v; ++i)
	{
	  memcpy (dest, src, srcRowbytes);
	  dest[m_bmp.rowbytes - 1] = 0;
	  src += srcRowbytes;
	  dest += m_bmp.rowbytes;
	}
    }
}

BitmapFont::~BitmapFont ()
{
  if (m_bmp.baseaddr[0] != NULL)
    {
      delete m_bmp.baseaddr[0];
    }
}

void BitmapFont::draw (GID gid, RECT * rect, COLOR fgColor, COLOR bgColor)
{
  COLOR color[2] = { bgColor, fgColor };
  CSPEC cspec = { 0x09, { 2 }, color };
  ERR err = gset_bcs (gid, &cspec);
  if (err != E_OK)
    {
      Error::outputMessageBV ("gset_bcs", err);
    }

  err = grst_bmp (gid, rect, &m_bmp, &m_bmp.bounds, NULL, G_STORE | G_CVFORM | G_CVCOLOR);
  if (err != E_OK)
    {
      Error::outputMessageBV ("grst_bmp", err);
    }
}

__EBI_END_NAMESPACE
