/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_char.h"
#include "ebi_device.h"

__EBI_BEGIN_NAMESPACE

Char::Char (TC code, FontType fontType)
{
  m_code = code;
  m_fontType = fontType;

  m_width = 0;
  m_height = 0;
  m_pnt = toPNT (0, 0);
}

Char::~Char ()
{
}

W Char::getWidth () const
{
  return m_width;
}

W Char::updateSize (Device* device)
{
  device->selectFont (m_fontType);
  m_width = device->getCharWidth (m_code);
  return m_width;
}

void Char::updateLayout (PNT* pnt, W max)
{
  if (max > 0 && pnt->x + m_width >= max)
    {
      pnt->x = 0;
      ++pnt->y;
    }

  m_pnt = *pnt;
  pnt->x += m_width;
}

void Char::draw (Device* device, W low, W high)
{
  if (m_pnt.y < low || m_pnt.y > high)
    {
      return;
    }
  if (m_pnt.x == 0)
    {
      device->newLine ();
    }

  device->selectFont (m_fontType);
  device->drawChar (m_code);
}

bool Char::isHit (const PNT& pnt) const
{
  if ((pnt.x >= m_pnt.x && pnt.x < m_pnt.x + m_width) && pnt.y == m_pnt.y)
    {
      return true;
    }
  return false;
}

void Char::getString (String* str) const
{
  *str += m_code;
}

__EBI_END_NAMESPACE
