/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_container.h"
#include "ebi_device.h"

__EBI_BEGIN_NAMESPACE

Container::Container ()
{
  m_width = 0;
  m_height = 0;
  m_low = 0;
  m_high = 0;
}

Container::~Container ()
{
  clear ();
}

W Container::getWidth () const
{
  return m_width;
}

W Container::updateSize (Device* device)
{
  m_width = 0;
  for (Elements::iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      m_width += (*ei)->updateSize (device);
    }
  return m_width;
}

void Container::updateLayout (PNT* pos, W max)
{
  m_low = pos->y;
  for (Elements::iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      (*ei)->updateLayout (pos, max);
    }
  m_high = pos->y;
}

void Container::draw (Device* device, W low, W high)
{
  if (m_low > high || m_high < low)
    {
      return;
    }

  for (Elements::iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      (*ei)->draw (device, low, high);
    }
}

bool Container::isHit (const PNT& pnt) const
{
  for (Elements::const_iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      if ((*ei)->isHit (pnt))
	{
	  return true;
	}
    }
  return false;
}

const Reference* Container::getReference (const PNT& pnt) const
{
  for (Elements::const_iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      const Reference* reference = (*ei)->getReference (pnt);
      if (reference != NULL)
	{
	  return reference;
	}
    }
  return NULL;
}

void Container::getString (String* str) const
{
  for (Elements::const_iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      (*ei)->getString (str);
    }
}

void Container::clear ()
{
  for (Elements::iterator ei = m_elements.begin (); ei != m_elements.end (); ++ei)
    {
      if (*ei != NULL)
	{
	  delete *ei;
	}
    }
  m_elements.clear ();
}

void Container::addElement (Element * element)
{
  m_elements.push_back (element);
}

__EBI_END_NAMESPACE
