/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001-2002  HASEGAWA Masafumi
 */

#include "ebi_error.h"

#define APP_NAME "ebi"

__EBI_BEGIN_NAMESPACE

EB_Error_Code Error::s_error = EB_SUCCESS;

Error::Error ()
{
}

void Error::outputMessage (const char *str)
{
  printf ("%s: %s\n", APP_NAME, str);
}

void Error::outputMessageEB (const char *str)
{
  printf ("%s: %s, %s\n", APP_NAME, str, eb_error_message (s_error));
}

void Error::outputMessageBV (const char *str, ERR err)
{
  printf ("%s: %s, (%d/%08x)\n", APP_NAME, str, err >> 16, err);
}

void Error::outputMessageHook (EB_Hook_Code code)
{
  const struct
  {
    EB_Hook_Code code;
    char* name;
  }
  c_hookNames[] =
  {
    { EB_HOOK_INITIALIZE, "EB_HOOK_INITIALIZE" }, 
    { EB_HOOK_BEGIN_NARROW, "EB_HOOK_BEGIN_NARROW" },
    { EB_HOOK_END_NARROW, "EB_HOOK_END_NARROW" },
    { EB_HOOK_BEGIN_SUBSCRIPT, "EB_HOOK_BEGIN_SUBSCRIPT" },
    { EB_HOOK_END_SUBSCRIPT, "EB_HOOK_END_SUBSCRIPT" },

    { EB_HOOK_SET_INDENT, "EB_HOOK_SET_INDENT" }, 
    { EB_HOOK_NEWLINE, "EB_HOOK_NEWLINE" },
    { EB_HOOK_BEGIN_SUPERSCRIPT, "EB_HOOK_BEGIN_SUPERSCRIPT" }, 
    { EB_HOOK_END_SUPERSCRIPT, "EB_HOOK_END_SUPERSCRIPT" },
    { EB_HOOK_BEGIN_NO_NEWLINE, "EB_HOOK_BEGIN_NO_NEWLINE" }, 

    { EB_HOOK_END_NO_NEWLINE, "EB_HOOK_END_NO_NEWLINE" },
    { EB_HOOK_BEGIN_EMPHASIS, "EB_HOOK_BEGIN_EMPHASIS" }, 
    { EB_HOOK_END_EMPHASIS, "EB_HOOK_END_EMPHASIS" },
    { EB_HOOK_BEGIN_CANDIDATE, "EB_HOOK_BEGIN_CANDIDATE" }, 
    { EB_HOOK_END_CANDIDATE_GROUP, "EB_HOOK_END_CANDIDATE_GROUP" },

    { EB_HOOK_END_CANDIDATE_LEAF, "EB_HOOK_END_CANDIDATE_LEAF" },
    { EB_HOOK_BEGIN_REFERENCE, "EB_HOOK_BEGIN_REFERENCE" }, 
    { EB_HOOK_END_REFERENCE, "EB_HOOK_END_REFERENCE" },
    { EB_HOOK_BEGIN_KEYWORD, "EB_HOOK_BEGIN_KEYWORD" }, 
    { EB_HOOK_END_KEYWORD, "EB_HOOK_END_KEYWORD" },

    { EB_HOOK_NARROW_FONT, "EB_HOOK_NARROW_FONT" }, 
    { EB_HOOK_WIDE_FONT, "EB_HOOK_WIDE_FONT" },
    { EB_HOOK_ISO8859_1, "EB_HOOK_ISO8859_1" },
    { EB_HOOK_NARROW_JISX0208, "EB_HOOK_NARROW_JISX0208" }, 
    { EB_HOOK_WIDE_JISX0208, "EB_HOOK_WIDE_JISX0208" },

    { EB_HOOK_GB2312, "EB_HOOK_GB2312" },
    { EB_HOOK_BEGIN_MONO_GRAPHIC, "EB_HOOK_BEGIN_MONO_GRAPHIC" }, 
    { EB_HOOK_END_MONO_GRAPHIC, "EB_HOOK_END_MONO_GRAPHIC" },
    { EB_HOOK_BEGIN_GRAY_GRAPHIC, "EB_HOOK_BEGIN_GRAY_GRAPHIC" },
    { EB_HOOK_END_GRAY_GRAPHIC, "EB_HOOK_END_GRAY_GRAPHIC" },

    { EB_HOOK_BEGIN_COLOR_BMP, "EB_HOOK_BEGIN_COLOR_BMP" }, 
    { EB_HOOK_BEGIN_COLOR_JPEG, "EB_HOOK_BEGIN_COLOR_JPEG" }, 
    { EB_HOOK_END_COLOR_GRAPHIC, "EB_HOOK_END_COLOR_GRAPHIC" }, 
    { EB_HOOK_END_IN_COLOR_GRAPHIC, "EB_HOOK_END_IN_COLOR_GRAPHIC" }, 
    { EB_HOOK_BEGIN_WAVE, "EB_HOOK_BEGIN_WAVE" }, 

    { EB_HOOK_END_WAVE, "EB_HOOK_END_WAVE" }, 
    { EB_HOOK_BEGIN_MPEG, "EB_HOOK_BEGIN_MPEG" }, 
    { EB_HOOK_END_MPEG, "EB_HOOK_END_MPEG" }, 

    { EB_HOOK_BEGIN_GRAPHIC_REFERENCE, "EB_HOOK_BEGIN_GRAPHIC_REFERENCE" }, 
    { EB_HOOK_END_GRAPHIC_REFERENCE, "EB_HOOK_END_GRAPHIC_REFERENCE" }, 

    { EB_HOOK_GRAPHIC_REFERENCE, "EB_HOOK_GRAPHIC_REFERENCE" }, 

    { EB_HOOK_NULL }
  };

  const char* name = NULL;
  for (int i = 0; c_hookNames[i].code != EB_HOOK_NULL; ++i)
    {
      if (code == c_hookNames[i].code)
	{
	  name = c_hookNames[i].name;
	  break;
	}
    }
  if (name == NULL)
    {
      printf ("Unknown EB_Hook_Code=%d\n", code);
      return;
    }

  if (code == EB_HOOK_INITIALIZE)
    {
      printf ("\n");
    }
  printf ("%s\n", name);
  if (code == EB_HOOK_NEWLINE)
    {
      printf ("\n");
    }
}

__EBI_END_NAMESPACE
