/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_imagelink.h"
#include "ebi_device.h"
#include "ebi_imagepage.h"
#include "ebi_book.h"
#include "databox.h"

__EBI_BEGIN_NAMESPACE

ImageLink::ImageLink (Subbook* subbook, ImageType type, int width, int height)
  : Reference (subbook)
{
  m_type = type;
  m_width = width;
  m_height = height;
}

ImageLink::~ImageLink ()
{
}

Page* ImageLink::execute () const
{
  EB_Position pos;
  getPos (&pos);
  ImagePage* page = new ImagePage (getSubbook (), &pos);
  if (!getSubbook ()->getBook ()->getImagePage (pos, m_type, m_width, m_height, page))
    {
      delete page;
      page = NULL;
      errpanel (ER_IMAGE, 0);
    }
  return page;
}

ImageLink::ImageType ImageLink::getType () const
{
  return m_type;
}

int ImageLink::getWidth () const
{
  return m_width;
}

int ImageLink::getHeight () const
{
  return m_height;
}

__EBI_END_NAMESPACE
