/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_mainwindow.h"
#include "ebi.h"
#include "databox.h"
#include "ebi_device.h"
#include "ebi_page.h"
#include "ebi_reference.h"
#include "ebi_error.h"

__EBI_BEGIN_NAMESPACE

MainWindow::MainWindow ()
{
  m_pict = 0;
  m_wid = -1;
  m_bFullview = false;
  m_rect = toRECT (0, 0, 0, 0);
  m_rbar = -1;
  m_rbarVal.knobLow = 0;
  m_rbarVal.knobHigh = 1;
  m_rbarVal.low = 0;
  m_rbarVal.high = 1;
  m_device = NULL;
  m_page = NULL;
  m_reference = NULL;
}

MainWindow::~MainWindow ()
{
  if (m_device != NULL)
    {
      delete m_device;
    }
}

void MainWindow::onIdle (UW time, PNT pos, H cmd, H wid, UW stat)
{
  if (cmd == 0 && wid == m_wid && !(stat & ES_CMD))
    {
      gset_ptr (PS_SELECT, NULL, (COLOR) -1, (COLOR) -1);

      if (m_device != NULL && m_page != NULL)
	{
	  PNT pnt = m_device->calcPoint (pos, m_rbarVal.knobLow);
	  const Reference* reference = m_page->getReference (pnt);
	  if (reference != NULL && reference != m_reference)
	    {
	      String str;
	      reference->getString (&str);
	      pdsp_msg ((TC*) str.c_str ());

	      m_reference = reference;
	    }
	}
    }
}

W MainWindow::onMenu (UW time, PNT pos, H cmd, H wid, UW stat)
{
  return g_menu.select ();
}  

void MainWindow::onDisplay (W mode, const RECT* rect)
{
  if (m_device != NULL && wcolor_changed == 1)
    {
      m_device->updateFontSpec ();
    }

  switch (mode)
    {
    case -1: /* ɽ׵ ɽ= new=NULL */
    case 0: /* ɽ׵ᥤ٥ ɽ= new=NULL */
    case 7: /* ѥͥ ɽ= new=NULL */
    case 9: /* ѹ ɽ= new=ΰ */
      redraw (rect);
      break;

    case 1: /* ɥåư ɽ= new=NULL */
      {
	m_bFullview = false;
	WDSTAT ws;
	wget_sts (m_wid, &ws, NULL);
	m_rect = ws.r;

	redraw (rect);
      }
      break;

    case 2: /* ɥåѷ ɽ= new=ΰ */
    case 3: /* ɥåѷ ɽ= new=ΰ */
      {
	m_bFullview = false;
	WDSTAT ws;
	wget_sts (m_wid, &ws, NULL);
	m_rect = ws.r;

	updateDeviceRect ();
      }
      break;

    case 4: /* ̾⡼ڴ ɽ= new=ΰ */
    case 6: /* ̾⡼ڴ ɽ= new=ΰ */
      setFullview (false);
      break;

    case 5: /* ̥⡼ڴ ɽ= new=ΰ */
      setFullview (true);
      break;

    case 8: /* ѹ ɽ= new=ΰ */
      break;
    }
}

W MainWindow::onKey (UW time, PNT pos, H key, H ch, UW stat)
{
  W i = 1;
  if (stat & ES_CMD)
    {
      i = m_rbarVal.knobHigh - m_rbarVal.knobLow - 1;
    }

  W j;
  switch (ch)
    {
    case KC_CC_U: // ᥤãå 
      j = m_rbarVal.knobLow - m_rbarVal.low;
      if (j > 0)
	{
	  scroll (-(i < j ? i : j));
	  redraw ();
	}
      return 0;

    case KC_SC_U: // ֣ãå 
      j = m_rbarVal.knobLow - m_rbarVal.low;
      if (j > 0)
	{
	  scroll (-j);
	  redraw ();
	}
      return 0;

    case KC_CC_D: // ᥤãå 
      j = m_rbarVal.high - m_rbarVal.knobHigh;
      if (j > 0)
	{
	  scroll ((i < j ? i : j));
	  redraw ();
	}
      return 0;

    case KC_SC_D: // ֣ãå 
      j = m_rbarVal.high - m_rbarVal.knobHigh;
      if (j > 0)
	{
	  scroll (j);
	  redraw ();
	}
      return 0;
    }

  if ((wevt.s.type == EV_KEYDWN) && (stat & ES_CMD))
    {
      return g_menu.select ();
    }

  return 0;
}

W MainWindow::onPress (UW time, PNT pos, H cmd, UW stat)
{
  if (m_device == NULL ||  m_page == NULL)
    {
      return 0;
    }

  PNT pnt = m_device->calcPoint (pos, m_rbarVal.knobLow);
  const Reference* reference = m_page->getReference (pnt);
  if (reference != NULL)
    {
      openReferencePage (reference);
    }
  
  return 0;
}

W MainWindow::onFinish (W mode)
{
  switch (mode)
    {
    case 0:
    case 1:
      /* 䵤򹹿ƽλ */
      g_fusen.setFullview (m_bFullview);
      g_fusen.setWindowRect (m_rect);
      return 1;

    case 2:
      /* 䵤򹹿λ */
      return -1;
    }

  return 0;
}

void MainWindow::onScroll (W type, W diff)
{
  RECT vrect = toRECT (0, m_rbarVal.knobLow, 0, m_rbarVal.knobHigh);
  RECT frect = toRECT (0, m_rbarVal.low, 0, m_rbarVal.high);
  PNT pnt = chk_scroll (type, diff, 1, vrect, frect);
  if (pnt.y != 0)
    {
      scroll (-pnt.y);
    }
}  

void MainWindow::getWinfo (WINFOREC* winfo) const
{
  winfo->wid = m_wid;
  winfo->chgmod = 0x10 | W_MOVE;
  winfo->pid[SBAR_RIGHT] = m_rbar;
  winfo->pid[SBAR_BOTTOM] = -1;
  winfo->pid[SBAR_LEFT] = -1;
}

W MainWindow::open (W pwid, W pict, const TC* title)
{
  m_pict = pict;
  m_title = title;

  m_bFullview = g_fusen.getFullview ();
  m_rect = g_fusen.getWindowRect ();

  UW attr = WA_SIZE | WA_HHDL | WA_VHDL | WA_RBAR | WA_BGDSP;
  RECT rc = m_rect;
  m_wid = wopn_wnd (attr, pwid, &rc, NULL, m_pict, (TC*) m_title.c_str (), NULL, NULL);
  if (m_wid < 0)
    {
      Error::outputMessageBV ("wopn_wnd", m_wid);
      errpanel (ER_OPEN_WINDOW, m_wid);
      return m_wid;
    }
  if (m_bFullview)
    {
      wchg_wnd (m_wid, NULL, W_MOVE);
    }

  wget_bar (m_wid, &m_rbar, NULL, NULL);
  cchg_par (m_rbar, P_DRAGBREAK | P_NORMAL | P_ENABLE);

  m_device = new Device (wget_gid (m_wid));
  updateDeviceRect ();

  return m_wid;
}

void MainWindow::close ()
{
  if (m_wid > 0)
    {
      wcls_wnd (m_wid, CLR);
      m_wid = 0;
    }
}

void MainWindow::redraw (const RECT* rect)
{
  do
    {
      wsta_dsp (m_wid, NULL, NULL);
      wera_wnd (m_wid, NULL);

      if (m_page != NULL)
	{
	  m_page->draw (m_rbarVal.knobLow, m_rbarVal.knobHigh - 1);
	}
    }
  while (wend_dsp (m_wid) > 0);
}

bool MainWindow::getFullview () const
{
  return m_bFullview;
}

void MainWindow::setFullview (bool bFullview)
{
  m_bFullview = bFullview;
      
  updateDeviceRect ();
}

void MainWindow::changeFullview ()
{
  ERR err = wchg_wnd (m_wid, NULL, W_MOVE);
  setFullview ((err == 1));
}

void MainWindow::requestDisplay ()
{
  wreq_dsp (m_wid);
}

void MainWindow::setPage (Page* page)
{
  m_page = page;

  String str = m_title;
  if (m_page != NULL)
    {
      m_page->setDevice (m_device);
      updateLayout ();

      Subbook* subbook = m_page->getSubbook ();
      if (subbook != NULL)
	{
	  str += TK_COLN;
	  str += subbook->getTitle ();
	}
    }

  std::replace (str.begin (), str.end (), TK_SP0, TK_KSP);
  std::replace (str.begin (), str.end (), TK_SP1, TK_KSP);
  
  wset_tit (m_wid, m_pict, (TC*) str.c_str (), 0);
}

void MainWindow::updateDeviceRect ()
{
  RECT rect;
  wget_wrk (m_wid, &rect);
  if (rect.c.top != 0 || rect.c.left != 0)
    {
      rect.c.left = 0;
      rect.c.top = 0;
      wset_wrk (m_wid, &rect);
    }
  
  if (m_device != NULL)
    {
      m_device->setRect (rect);
    }

  updateLayout ();
}

void MainWindow::updateLayout ()
{
  m_rbarVal.knobLow = 0;
  m_rbarVal.high = 1;
  if (m_page != NULL)
    {
      m_rbarVal.high = m_page->updateLayout ();
    }

  updateScrollBar ();
}

void MainWindow::updateScrollBar ()
{
  W lines = 0;
  if (m_device != NULL)
    {
      lines = m_device->calcLines ();
    }
  if (lines < 1)
    {
      lines = 1;
    }
  if (m_rbarVal.high < lines)
    {
      m_rbarVal.high = lines;
    }
  m_rbarVal.knobHigh = m_rbarVal.knobLow + lines;
  if (m_rbarVal.knobHigh > m_rbarVal.high)
    {
      m_rbarVal.knobLow -= (m_rbarVal.knobHigh - m_rbarVal.high);
      m_rbarVal.knobHigh = m_rbarVal.high;
    }
  cset_val (m_rbar, 4, (W*) &m_rbarVal);

  if (m_device != NULL)
    {
      m_device->clear ();
      redraw ();
    }
}

void MainWindow::scroll (W diff)
{
  m_rbarVal.knobLow += diff;
  updateScrollBar ();
}

__EBI_END_NAMESPACE



