/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#ifndef __EBI_MAINWINDOW_H
#define __EBI_MAINWINDOW_H

#include "ebi_config.h"
#include "ebi_window.h"
#include "ebi_string.h"

__EBI_BEGIN_NAMESPACE

class Device;
class Page;
class Reference;

class MainWindow : public Window
{
 public:
  MainWindow ();
  virtual ~MainWindow ();

  virtual void onIdle (UW time, PNT pos, H cmd, H wid, UW stat);
  virtual W onMenu (UW time, PNT pos, H cmd, H wid, UW stat);
  virtual void onDisplay (W mode, const RECT* rect);
  virtual W onKey (UW time, PNT pos, H key, H ch, UW stat);
  virtual W onPress (UW time, PNT pos, H cmd, UW stat);
  virtual W onFinish (W mode);
  virtual void onScroll (W type, W diff);
  W getWid () const;
  W open (W pwid, W pict, const TC* title);
  void close ();
  void redraw (const RECT* rect = NULL);
  bool getFullview () const;
  void setFullview (bool bFullview);
  void changeFullview ();
  void requestDisplay ();
  void setPage (Page* page);
  void updateLayout ();

 protected:
  virtual void getWinfo (WINFOREC* winfo) const;

 private:
  enum
  {
    SBAR_RIGHT, /* С */
    SBAR_BOTTOM, /* С */
    SBAR_LEFT, /* С */
  };

  W m_pict;
  String m_title;
  W m_wid;
  bool m_bFullview;
  RECT m_rect;
  W m_rbar;
  struct
  {
    W knobLow; /* Υ֤κǾ */
    W knobHigh; /* Υ֤κ */
    W low; /* ΤκǾ */
    W high; /* Τκ */
  }
  m_rbarVal;
  Device* m_device;
  Page* m_page;
  const Reference* m_reference;

  void updateDeviceRect ();
  void updateScrollBar ();
  void scroll (W diff);
};
  
__EBI_END_NAMESPACE

#endif /* __EBI_MAINWINDOW_H */
