/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001-2002  HASEGAWA Masafumi
 */

#include "ebi_menu.h"
#include "ebi.h"
#include "databox.h"
#include "ebi_error.h"
#include "ebi_subbook.h"
#include "ebi_textpage.h"
#include "ebi_keywordpanel.h"

__EBI_BEGIN_NAMESPACE

FUNCP Menu::s_funcs[MAX_MENU];

Menu::Menu ()
{
}

Menu::~Menu ()
{
}

W Menu::getInterNo (W index)
{
  return (index - MENU_EXIT);
}

W Menu::getActMask (W index)
{
  return (UW)(1L << getInterNo (index));
}

W Menu::onTool (W para)
{
  setpointer (PS_BUSY, NULL);
  if (oexe_apg (0, para) > 0)
    {
      setpointer (0x8001, NULL); /* ݥ󥿥å */
    }
  return 0;
}

W Menu::onExit (W para)
{
  return g_window.onFinish (1);
}

W Menu::onFullview (W para)
{
  g_window.changeFullview ();
  return 0;
}

W Menu::onRedisp (W para)
{
  g_window.requestDisplay ();
  return 0;
}

W Menu::onWordSearch (W para)
{
  static String s_keyword;
  Book::SearchMethod search = g_fusen.getSearchMethod ();
  KeywordPanel keywordPanel;
  keywordPanel.setKeyword (s_keyword);
  keywordPanel.setSearchMethod (search);
  if (keywordPanel.doPanel () != KeywordPanel::PNL_MS_SET)
    {
      return 0;
    }
  keywordPanel.getKeyword (&s_keyword);
  search = keywordPanel.getSearchMethod ();
  g_fusen.setSearchMethod (search);

  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);

  TextPage* page = new TextPage (NULL, NULL, true);
  if (g_book.getEntryPageAll (s_keyword.c_str (), search, page))
    {
      g_pageSet.insert (page);
    }
  else
    {
      delete page;
    }

  return 0;
}

W Menu::onMenuSearch (W para)
{
  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);

  Subbook* subbook = g_pageSet.getCurrPage ()->getSubbook ();
  Book* book = subbook->getBook ();
  if (book->selectSubbook (subbook->getSubbook ()))
    {
      EB_Position pos;
      if (book->getMenu (&pos))
	{
	  TextPage* page = new TextPage (subbook, &pos);
	  if (book->getTextPage (pos, page))
	    {
	      g_pageSet.insert (page);
	    }
	  else
	    {
	      delete page;
	    }
	}
    }
 
  return 0;
}

W Menu::onCopyright (W para)
{
  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);

  Subbook* subbook = g_pageSet.getCurrPage ()->getSubbook ();
  Book* book = subbook->getBook ();
  if (book->selectSubbook (subbook->getSubbook ()))
    {
      EB_Position pos;
      if (book->getCopyright (&pos))
	{
	  TextPage* page = new TextPage (subbook, &pos);
	  if (book->getTextPage (pos, page))
	    {
	      g_pageSet.insert (page);
	    }
	  else
	    {
	      delete page;
	    }
	}
    }

  return 0;
}


W Menu::onSubbookTitle (W para)
{
  openSubbookTitlePage ();

  return 0;
}

W Menu::onPrevPage (W para)
{
  g_pageSet.prevPage ();
  return 0;
}

W Menu::onNextPage (W para)
{
  g_pageSet.nextPage ();
  return 0;
}

#if 0
W Menu::onForwardPage (W para)
{
  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);

  Subbook* subbook = g_pageSet.getCurrPage ()->getSubbook ();
  Book* book = subbook->getBook ();
  if (book->selectSubbook (subbook->getSubbook ()))
    {
      EB_Position pos;
      if (book->getForwardPage (&pos))
	{
	  TextPage* page = new TextPage (subbook, &pos);
	  if (book->getTextPage (pos, page))
	    {
	      g_pageSet.insert (page);
	    }
	  else
	    {
	      delete page;
	    }
	}
    }
  return 0;
}

W Menu::onBackwardPage (W para)
{
  gset_ptr (PS_BUSY, NULL, (COLOR) -1, (COLOR) -1);

  Subbook* subbook = g_pageSet.getCurrPage ()->getSubbook ();
  Book* book = subbook->getBook ();
  if (book->selectSubbook (subbook->getSubbook ()))
    {
      EB_Position pos;
      if (book->getBackwardPage (&pos))
	{
	  TextPage* page = new TextPage (subbook, &pos);
	  if (book->getTextPage (pos, page))
	    {
	      g_pageSet.insert (page);
	    }
	  else
	    {
	      delete page;
	    }
	}
    }
  return 0;
}
#endif

bool Menu::open ()
{
  s_funcs[MENU_TOOL] = (FUNCP) onTool;
  s_funcs[MENU_EXIT] = (FUNCP) onExit;
  s_funcs[MENU_FULLVIEW] = (FUNCP) onFullview;
  s_funcs[MENU_REDISP] = (FUNCP) onRedisp;
  s_funcs[MENU_WORDSEARCH] = (FUNCP) onWordSearch;
  s_funcs[MENU_MENUSEARCH] = (FUNCP) onMenuSearch;
  s_funcs[MENU_COPYRIGHT] = (FUNCP) onCopyright;
  s_funcs[MENU_SUBBOOKTITLE] = (FUNCP) onSubbookTitle;
  s_funcs[MENU_PREVPAGE] = (FUNCP) onPrevPage;
  s_funcs[MENU_NEXTPAGE] = (FUNCP) onNextPage;
#if 0
  s_funcs[MENU_FORWARDPAGE] = (FUNCP) onForwardPage;
  s_funcs[MENU_BACKWARDPAGE] = (FUNCP) onBackwardPage;
#endif

  ERR err = openmenu (MN_MAIN);
  if (err < E_OK)
    {
      Error::outputMessageBV ("openmenu", err);
      errpanel (ER_OPEN_MENU, err);
      return false;
    }

  return true;
}

W Menu::select ()
{
  // 󥸥
  indmenu (getInterNo (MENU_FULLVIEW), g_window.getFullview (), 0);

  // ǽ
  UW inact = 0;
  Subbook* subbook = g_pageSet.getCurrPage ()->getSubbook ();
  if (subbook == NULL || !subbook->haveMenu ())
    {
      inact |= getActMask (MENU_MENUSEARCH);
    }
  if (subbook == NULL || !subbook->haveCopyright ())
    {
      inact |= getActMask (MENU_COPYRIGHT);
    }
  if (g_pageSet.getPrevPage () == NULL)
    {
      inact |= getActMask (MENU_PREVPAGE);
    }
  if (g_pageSet.getNextPage () == NULL)
    {
      inact |= getActMask (MENU_NEXTPAGE);
    }
#if 0
  if (!subbook->haveForwardPage ())
    {
      inact |= getActMask (MENU_FORWARDPAGE);
    }
  if (!subbook->haveBackwardPage ())
    {
      inact |= getActMask (MENU_BACKWARDPAGE);
    }
#endif
  inactmenu (inact, 0);

  return selmenu (0, s_funcs);
}

__EBI_END_NAMESPACE
