/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001-2002  HASEGAWA Masafumi
 */

#include "ebi_pageset.h"
#include "ebi.h"
#include "ebi_page.h"

__EBI_BEGIN_NAMESPACE

PageSet::PageSet ()
{
  m_current = -1;
}

PageSet::~PageSet ()
{
  for (Pages::iterator pi = m_pages.begin (); pi != m_pages.end (); ++pi)
    {
      if (*pi != NULL)
	{
	  delete *pi;
	}
    }
  m_pages.clear ();
}

Page* PageSet::getCurrPage () const
{
  return m_pages[m_current];
}

Page* PageSet::getPrevPage () const
{
  if (m_current == 0)
    {
      return NULL;
    }
  return m_pages[m_current - 1];
}

Page* PageSet::getNextPage () const
{
  if (m_current == (int) (m_pages.size () - 1))
    {
      return NULL;
    }
  return m_pages[m_current + 1];
}

void PageSet::insert (Page* page)
{
  if (m_pages.size () == 0)
    {
      m_current = 0;
      m_pages.push_back (page);
    }
  else
    {
      int last = m_pages.size () - 1;
      for (int i = last; i > m_current; --i)
	{
	  delete m_pages[i];
	  m_pages.pop_back ();
	}

      if (m_pages.size () == MAX_PAGE)
	{
	  delete *m_pages.begin ();
	  m_pages.erase (m_pages.begin ());
	  --m_current;
	}

      m_pages.push_back (page);
      ++m_current;
    }

  g_window.setPage (m_pages[m_current]);
}

void PageSet::prevPage ()
{
  if (getPrevPage () == NULL)
    {
      return;
    }
  --m_current;

  g_window.setPage (m_pages[m_current]);
}

void PageSet::nextPage ()
{
  if (getNextPage () == NULL)
    {
      return;
    }
  ++m_current;

  g_window.setPage (m_pages[m_current]);
}

__EBI_END_NAMESPACE
