/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_sound.h"
#include "ebi_error.h"
#include "databox.h"

__EBI_BEGIN_NAMESPACE

const W c_bufferSize = 64 * 1024; // 64KХ
const W c_bufferDiv = 8;

Sound::Sound ()
{
  m_dd = 0;
  memset (&m_mode, 0, sizeof (m_mode));
  m_delay = 0;
}

Sound::~Sound ()
{
  close ();
}

bool Sound::open (const SDPcmMode& mode)
{
  close ();
  m_mode = mode;

  // ɥǥХΥץ
  m_dd = opn_dev (getdbox_text (TX_SOUND), D_READ | D_WRITE, NULL);
  if (m_dd < 0)
    {
      Error::outputMessageBV ("ɥǥХΥץ", m_dd);
      return false;
    }

  // PCMХåե
  W size = c_bufferSize;
  W a_size;
  ERR err = wri_dev (m_dd, DN_SDPCMBUFSZ, (B*) &size, sizeof (size), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("PCMХåե", err);
      return false;
    }

  // PCMư⡼ɤ
  err = wri_dev (m_dd, DN_SDPCMMODE, (B*) &m_mode, sizeof (m_mode), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("PCMư⡼ɤ", err);
      return false;
    }

  // 쥯
  SDSel selector;
  err = rea_dev (m_dd, DN_SDSELOUT, (B*) &selector, sizeof (selector), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("쥯", err);
      return false;
    }
  selector.pcm = True;
  err = wri_dev (m_dd, DN_SDSELOUT, (B*) &selector, sizeof (selector), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("쥯", err);
      return false;
    }

  // ܥ塼
  SDVol volume;
  memset (&volume, 0, sizeof (volume));
  volume.lv = 0xFFFF; // 
  volume.rv = 0xFFFF; // 
  err = wri_dev (m_dd, DN_SDVOLPCM, (B*) &volume, sizeof (volume), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("ܥ塼", err);
      return false;
    }

  // PCMߥȤ
  W count = 0; // 
  err = wri_dev (m_dd, DN_SDPCMENDCNT, (B*) &count, sizeof (count), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("PCMߥȤ", err);
      return false;
    }

  // 
  const W c_sampling[] = { 8000, 11025, 16000, 22050, 44100, 48000 };
  W bps = c_sampling[m_mode.sampling];
  if (mode.datafmt == PCMFmt16)
    {
      bps *= 2;
    }
  if (mode.stereo == True)
    {
      bps *= 2;
    }
  m_delay = (c_bufferSize / c_bufferDiv) * 1000 / bps;

  return true;
}

void Sound::close ()
{
  if (m_dd != 0)
    {
      // 
      SDPcmCtl ctl = PCM_STOP;
      W a_size;
      ERR err = wri_dev (m_dd, DN_SDPCMCTL, (B*) &ctl, sizeof (ctl), &a_size, NULL);
      if (err != E_OK)
	{
	  Error::outputMessageBV ("", err);
	}

      // ɥǥХΥ
      err = cls_dev (m_dd, 0, NULL);
      if (err != E_OK)
	{
	  Error::outputMessageBV ("ɥǥХΥ", err);
	}
      m_dd = 0;
    }
}

bool Sound::output (const char* binary, int length)
{
  // ǡκǽν񤭹
  W size = length;
  if (size > c_bufferSize)
    {
      size = c_bufferSize;
    }
  W a_size;
  ERR err = wri_dev (m_dd, 0, (B*) binary, size, &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("ǡκǽν񤭹", err);
      return false;
    }
  W pos = size;

  // 򳫻
  SDPcmCtl ctl = PCM_PLAY;
  err = wri_dev (m_dd, DN_SDPCMCTL, (B*) &ctl, sizeof (ctl), &a_size, NULL);
  if (err != E_OK)
    {
      Error::outputMessageBV ("򳫻", err);
      return false;
    }

  while (pos < length)
    {
      while (true)
	{
	  ERR err = wri_dev (m_dd, 1, NULL, 0, &a_size, NULL);
	  if (err < 0)
	    {
	      Error::outputMessageBV ("Хåեζɤ߹", err);
	      return false;
	    }

	  if (a_size >= (c_bufferSize / c_bufferDiv))
	    {
	      break;
	    }
	  dly_tsk (m_delay);
	}

      // ǡν񤭹
      size = length - pos;
      if (size > (c_bufferSize / c_bufferDiv))
	{
	  size = (c_bufferSize / c_bufferDiv);
	}
      err = wri_dev (m_dd, 1, (B*) (binary + pos), size, &a_size, NULL);
      if (err != E_OK)
	{
	  Error::outputMessageBV ("ǡν񤭹", err);
	  return false;
	}
      pos += size;
    }

  return true;
}

__EBI_END_NAMESPACE
