/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_soundlink.h"
#include "ebi_wave.h"
#include "ebi_book.h"
#include "ebi_sound.h"
#include "databox.h"

__EBI_BEGIN_NAMESPACE

SoundLink::SoundLink (Subbook* subbook, int size)
  : Reference (subbook)
{
  m_size = size;
}

SoundLink::~SoundLink ()
{
}

Page* SoundLink::execute () const
{
  EB_Position pos;
  getPos (&pos);
  Wave wave;
  if (!getSubbook ()->getBook ()->getSoundData (pos, m_size, &wave))
    {
      errpanel (ER_SOUND, 0);
      return NULL;
    }
  
  SDPcmMode mode;
  if (!wave.getPcmMode (&mode))
    {
      errpanel (ER_WAVE, 0);
      return NULL;
    }
  int length;
  const char* binary = wave.getData (&length);

  Sound sound;
  if (sound.open (mode))
    {
      sound.output (binary, length);
      sound.close ();
    }
  
  return NULL;
}

__EBI_END_NAMESPACE
