/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2001  HASEGAWA Masafumi
 */

#include "ebi_textpage.h"
#include "ebi_device.h"

__EBI_BEGIN_NAMESPACE

TextPage::TextPage (Subbook* subbook, const EB_Position* pos, bool bNoFold)
  : Page (subbook, pos)
{
  m_bNoFold = bNoFold;
  m_bModified = false;
}

TextPage::~TextPage ()
{
  clear ();
}

W TextPage::updateLayout ()
{
  if (m_device == NULL)
    {
      return 0;
    }

  W max = 0;
  if (!m_bNoFold)
    {
      max = m_device->calcMax ();
    }

  PNT pos = { 0, 0 };
  for (Lines::iterator li = m_lines.begin (); li != m_lines.end (); ++li)
    {
      if (m_bModified)
	{
	  (*li)->updateSize (m_device);
	}

      (*li)->updateLayout (&pos, max);
    }
  m_bModified = false;

  return pos.y + 1;
}

void TextPage::draw (W low, W high)
{
  if (m_device == NULL)
    {
      return;
    }

  m_device->newPage ();
  for (Lines::iterator li = m_lines.begin (); li != m_lines.end (); ++li)
    {
      (*li)->draw (m_device, low, high);
    }
}

const Reference* TextPage::getReference (const PNT& pnt) const
{
  for (Lines::const_iterator li = m_lines.begin (); li != m_lines.end (); ++li)
    {
      const Reference* reference = (*li)->getReference (pnt);
      if (reference != NULL)
	{
	  return reference;
	}
    }
  return NULL;
}

void TextPage::clear ()
{
  for (Lines::iterator li = m_lines.begin (); li != m_lines.end (); ++li)
    {
      if (*li != NULL)
	{
	  delete *li;
	}
    }
  m_lines.clear ();
  m_bModified = true;
}

void TextPage::addLine (Line * line)
{
  m_lines.push_back (line);
  m_bModified = true;
}

void TextPage::deleteLine (Line * line)
{
  Lines::iterator li = std::find (m_lines.begin (), m_lines.end (), line);
  if (li != m_lines.end ())
    {
      m_lines.erase (li);
      delete line;
    }
}

__EBI_END_NAMESPACE
