/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2000-2001  HASEGAWA Masafumi
 */

#include <sys/stat.h>

#include "ebword.h"

// eb_search_word/eb_search_endword/eb_search_exactwordδؿɥ쥹
typedef int (*Search_Function) (EB_Book*, const char*);

// ܤ򤹤
static BOOL
select_subbook (Book* book, int subbook_index)
{
  if (subbook_index != ALL_SUBBOOK && (subbook_index < 0 || subbook_index >= book->subbook_count))
    {
      return FALSE;
    }

  g_error = eb_set_subbook (&book->book, book->subbooks[subbook_index]);
  if (g_error != EB_SUCCESS)
    {
      output_error_message ("ܤǤޤ");
      return FALSE;
    }

  if (eb_is_appendix_bound (&book->appendix))
    {
      g_error = eb_set_appendix_subbook (&book->appendix, book->subbooks[subbook_index]);
      if (g_error != EB_SUCCESS)
	{
	  output_error_message ("ϿǡǤޤ");
	  return FALSE;
	}
    }

  return TRUE;
}

// ᥽åɤμ
static Search_Function
get_search_function (Book* book, const char* word, char* buffer)
{
  int len;

  len = strlen (word);
  if (word[0] == '*')
    {
      strcpy (buffer, word + 1);
      return eb_search_endword;
    }
  else if (word[len - 1] == '*')
    {
      strncpy (buffer, word, len - 1);
      buffer[len - 1] = '\0';
      return eb_search_word;
    }

  strcpy (buffer, word);
  return eb_search_exactword;
}

// ⡼
static BOOL
exec_search (Book* book, int subbook_index, const char* word, BOOL heading_only)
{
  Search_Function search_function;
  char buffer[EB_MAX_WORD_LENGTH + 1];
  int i;

  search_function = get_search_function (book, word, buffer);

  for (i = 0; i < book->subbook_count; ++i)
    {
      if (subbook_index != ALL_SUBBOOK && i != subbook_index)
	{
	  continue;
	}
      if (!select_subbook (book, i))
	{
	  continue;
	}

      g_error = (*search_function) (&book->book, buffer);
      if (g_error == EB_SUCCESS)
	{
	  output_search (book, heading_only);
	}
    }

  return TRUE;
}

// ⡼
static BOOL
exec_copyright (Book* book, int subbook_index)
{
  int i;

  for (i = 0; i < book->subbook_count; ++i)
    {
      if (subbook_index != ALL_SUBBOOK && i != subbook_index)
	{
	  continue;
	}
      if (!select_subbook (book, i))
	{
	  continue;
	}

      output_title (book, i);
      output_copyright (book);
    }

  return TRUE;
}

// ȥ⡼
static BOOL
exec_title (Book* book, int subbook_index)
{
  int i;

  for (i = 0; i < book->subbook_count; ++i)
    {
      if (subbook_index != ALL_SUBBOOK && i != subbook_index)
	{
	  continue;
	}

      output_title (book, i);
    }

  return TRUE;
}

// ϿǡΥѥ̾
static BOOL
get_appendix_path (const char* book_path, const char* appendix_path, char* buffer)
{
  const char * c_names[] =
    {
      "_APPENDIX",
      "_appendix",
      "_Appendix",
      NULL
    };
  int len, i;
  struct stat buf;

  if (appendix_path[0] != '\0')
    {
      strcpy (buffer, appendix_path);
      return TRUE;
    }
  
  for (i = 0; c_names[i] != NULL; ++i)
    {
      buffer[0] = '\0';
      if (book_path[0] != '\0')
	{
	  len = strlen (c_names[i]) + 1;
	  if (L_PATHNM - len < strlen (book_path))
	    {
	      return FALSE;
	    }
	  strcpy (buffer, book_path);
	  strcat (buffer, "/");
	}
      strcat (buffer, c_names[i]);

      if (stat (buffer, &buf) == 0)
	{
	  return TRUE;
	}
    }

  buffer[0] = '\0';
  return FALSE;
}

// ϽԤ
static BOOL
initialize (Book* book, const Params* params)
{
  char appendix_path[L_PATHNM + 1];

  eb_initialize_library();

  book->hookset = initialize_hookset (params->gaiji_code);

  eb_initialize_book (&book->book);

  if (eb_bind (&book->book, params->book_path) == -1)
    {
      output_error_message ("ҤǤޤ");
      return FALSE;
    }

  eb_initialize_all_subbooks (&book->book);

  g_error = eb_subbook_list (&book->book, book->subbooks, &book->subbook_count);
  if (g_error != EB_SUCCESS)
    {
      output_error_message ("ܤǤޤ");
      return FALSE;
    }

  eb_initialize_appendix (&book->appendix);

  if (get_appendix_path (params->book_path, params->appendix_path, appendix_path))
    {
      g_error = eb_bind_appendix (&book->appendix, appendix_path);
      if (g_error != EB_SUCCESS)
	{
	  output_error_message ("ϿǡǤޤ");
	  return FALSE;
	}

      eb_load_all_appendix_subbooks (&book->appendix);
    }

  return TRUE;
}

// λԤ
static void
terminate (Book* book)
{
  if (eb_is_bound (&book->book))
    {
      if (eb_is_appendix_bound (&book->appendix))
	{
	  eb_finalize_appendix (&book->appendix);
	}

      eb_finalize_book (&book->book);
    }

  terminate_hookset ();

  eb_finalize_library();
}

// mainؿ
int
main (int argc, char *const *argv)
{
  Book book;
  Params params;

  if (!get_params (argc, argv, &params))
    {
      return E_OK;
    }

  if (initialize (&book, &params))
    {
      switch (params.exec_mode)
	{
	case MODE_SEARCH:
	  exec_search (&book, params.subbook_index, params.word, params.heading_only);
	  break;

	case MODE_COPYRIGHT:
	  exec_copyright (&book, params.subbook_index);
	  break;

	case MODE_TITLE:
	  exec_title (&book, params.subbook_index);
	  break;
	}
    }
  terminate (&book);

  return E_OK;
}
